@@ -6,17 +6,15 @@ from typing import Callable
 from typing import Dict
 from typing import List
 
-from PyQt5.QtCore import QMetaObject
-from PyQt5.QtCore import QSize
-from PyQt5.QtCore import Qt
 from PyQt5.QtCore import QObject
 from PyQt5.QtCore import QRunnable
+from PyQt5.QtCore import QSize
 from PyQt5.QtCore import QThreadPool
+from PyQt5.QtCore import Qt
 from PyQt5.QtCore import pyqtSignal
 from PyQt5.QtCore import pyqtSlot
-from PyQt5.QtGui import QIcon
 from PyQt5.QtGui import QPainter
-from PyQt5.QtGui import QPixmap
+from PyQt5.QtGui import QTextCursor
 from PyQt5.QtSvg import QSvgRenderer
 from PyQt5.QtWidgets import QFrame
 from PyQt5.QtWidgets import QGroupBox
@@ -25,13 +23,14 @@ from PyQt5.QtWidgets import QLabel
 from PyQt5.QtWidgets import QLineEdit
 from PyQt5.QtWidgets import QMainWindow
 from PyQt5.QtWidgets import QPushButton
+from PyQt5.QtWidgets import QSizePolicy
+from PyQt5.QtWidgets import QTextEdit
 from PyQt5.QtWidgets import QVBoxLayout
 from PyQt5.QtWidgets import QWidget
 
-from .exec import get_arguments
 from .exec import Argument
 from .exec import ArgumentKind
-
+from .exec import get_arguments
 
 HERE = Path(__file__).parent.resolve()
 
@@ -41,7 +40,7 @@ class VLine(QFrame):
 
     def __init__(self):
         super().__init__()
-        self.setFrameShape(self.VLine | self.Sunken)
+        self.setFrameShape(QFrame.VLine | QFrame.Sunken)
 
 
 class HLine(QFrame):
@@ -49,7 +48,9 @@ class HLine(QFrame):
 
     def __init__(self):
         super().__init__()
-        self.setFrameShape(self.HLine | self.Sunken)
+        self.setLineWidth(0)
+        self.setMidLineWidth(1)
+        self.setFrameShape(QFrame.HLine | QFrame.Sunken)
 
 
 class FunctionThreadSignals(QObject):
@@ -92,6 +93,27 @@ class FunctionRunnable(QRunnable):
         QThreadPool.globalInstance().start(self)
 
 
+class ConsoleOutput(QTextEdit):
+    def __init__(self, parent=None):
+        super().__init__(parent)
+        self.setReadOnly(True)
+        self.setLineWrapMode(QTextEdit.NoWrap)
+        self.insertPlainText("")
+
+    @pyqtSlot(str)
+    def append(self, text):
+        self.moveCursor(QTextCursor.End)
+        current = self.toPlainText()
+
+        if current == "":
+            self.insertPlainText(text)
+        else:
+            self.insertPlainText("\n" + text)
+
+        sb = self.verticalScrollBar()
+        sb.setValue(sb.maximum())
+
+
 class IconLabel(QLabel):
 
     IconSize = QSize(16, 16)
@@ -217,19 +239,33 @@ class View(QMainWindow):
 
         self.setWindowTitle("Contingency GUI")
 
-        # self.setGeometry(300, 300, 300, 200)
+        # self.setGeometry(300, 300, 500, 200)
 
         # The main frame in which all the other frames are located, the outer Application frame
 
         self.app_frame = QFrame()
         self.app_frame.setObjectName("AppFrame")
 
+        # We don't want this QFrame to shrink below 500 pixels, therefore set a minimum horizontal size
+        # and set the policy such that it can still expand from this minimum size. This will be used
+        # when we use adjustSize after replacing the arguments panel.
+
+        self.app_frame.setMinimumSize(500, 0)
+        sp = self.app_frame.sizePolicy()
+        sp.setHorizontalPolicy(QSizePolicy.MinimumExpanding)
+        self.app_frame.setSizePolicy(sp)
+
         self._layout_panels = QVBoxLayout()
         self._layout_buttons = QVBoxLayout()
 
         self._layout_panels.addLayout(self._layout_buttons)
         self._layout_panels.addWidget(HLine())
-        self._current_args_panel: QWidget = None
+        self._current_args_panel: QWidget = QWidget()
+        self._current_args_panel.hide()
+        self._layout_panels.addWidget(self._current_args_panel)
+        self._console_panel = ConsoleOutput()
+        self._layout_panels.addWidget(HLine())
+        self._layout_panels.addWidget(self._console_panel)
 
         self.app_frame.setLayout(self._layout_panels)
 
@@ -281,7 +317,7 @@ class View(QMainWindow):
 
     @pyqtSlot(object)
     def function_output(self, data: object):
-        print(data)
+        self._console_panel.append(str(data))
 
     @pyqtSlot()
     def function_complete(self):
